unit BusinessObject;

interface

Uses Global, DBTables, SysUtils;

type EEmptyVirtual = class(Exception);

	type TBusinessObject = class(TObject)
		private
			m_bIsModified: 	Boolean;
			m_bIsNew:				Boolean;

		public
			constructor Create;
			destructor Free; virtual;

			procedure SetNew;
			procedure ClearNew;
			function IsNew: Boolean;

			procedure SetModified;
			procedure ClearModified;
			function IsModified: Boolean;

			function IsEqual(obj: TBusinessObject): Boolean; virtual;
			procedure Assign(obj: TBusinessObject); virtual;

			function Load: Boolean; virtual;
			function Save: Boolean; virtual;
			function DBDelete: Boolean; virtual;

			function GetNextKey(table: string): Longint;
			function GetTempTableVariable(table: string): TTable;
end;

implementation

constructor TBusinessObject.Create;
begin
	inherited Create;

	m_bIsModified := False;
	m_bIsNew := True;
end;

destructor TBusinessObject.Free;
begin
	raise EEmptyVirtual.Create('Free');
end;


function TBusinessObject.IsEqual(obj: TBusinessObject): Boolean;
begin
	Result := True;

	if m_bIsModified <> obj.IsModified then
		Result := False;

	if m_bIsNew <> obj.IsNew then
		Result := False;
end;

procedure TBusinessObject.Assign(obj: TBusinessObject);
begin
	m_bIsModified := obj.IsModified;
	m_bIsNew := obj.IsNew;
end;

procedure TBusinessObject.SetModified;
begin
	m_bIsModified := True;
end;

procedure TBusinessObject.ClearModified;
begin
	m_bIsModified := False;
end;


function TBusinessObject.IsModified: Boolean;
begin
	Result := m_bIsModified;
end;

procedure TBusinessObject.SetNew;
begin
	m_bIsNew := True;
end;

procedure TBusinessObject.ClearNew;
begin
	m_bIsNew := False;
end;

function TBusinessObject.IsNew: Boolean;
begin
	Result := m_bIsNew;
end;

function TBusinessObject.GetNextKey(table: string): Longint;
begin
	if KeyTable.FindKey([table]) then
	begin
		Result := KeyTable.FindField('NextKey').AsInteger;

		KeyTable.Edit;
		KeyTable.FindField('NextKey').AsInteger := Result + 1;
		KeyTable.Post;
	end
	else
		Result := -1;
end;

function TBusinessObject.GetTempTableVariable(table: string): TTable;
var T: TTable;
begin
	T := TTable.Create(nil);

	with T do
	begin
		DatabaseName := CDatabaseName;			{ Set to local database alias }
		TableName := table;
		Open;
		First;
	end;

	Result := T;
end;

function TBusinessObject.Load: Boolean;
begin
	raise EEmptyVirtual.Create('Load');
end;

function TBusinessObject.Save: Boolean;
begin
	raise EEmptyVirtual.Create('Save');
end;

function TBusinessObject.DBDelete: Boolean;
begin
	raise EEmptyVirtual.Create('DBDelete');
end;

end.
